/*******************************************************************************
 * Copyright (c) 2010, 2024 BestSolution.at and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     Tom Schindl <tom.schindl@bestsolution.at> - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.emf.xpath;

import java.util.Iterator;
import java.util.Spliterator;
import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import org.apache.commons.jxpath.util.TypeUtils;

/**
 * Context in which the xpath is executed
 *
 * @since 1.0
 */
public interface XPathContext {

	/**
	 * Evaluates the xpath and returns the resulting object. Primitive types are
	 * wrapped into objects.
	 *
	 * @param xpath
	 *            to evaluate
	 * @return Object found
	 */
	Object getValue(String xpath);

	/**
	 * Evaluates the xpath, converts the result to the specified class and
	 * returns the resulting object.
	 *
	 * @param xpath
	 *            to evaluate
	 * @param requiredType
	 *            required type
	 * @return Object found
	 */
	<T> T getValue(String xpath, Class<T> requiredType);

	/**
	 * Traverses the xpath and returns an Iterator of all results found for the
	 * path. If the xpath matches no properties in the graph, the Iterator will be
	 * empty, but not null.
	 *
	 * @param <T>   the expected object type
	 *
	 * @param xpath to iterate
	 * @return Iterator&lt;Object&gt;
	 */
	<T> Iterator<T> iterate(String xpath);

	/**
	 * Traverses the xpath and returns an {@link Stream} of all results found for
	 * the path. If the xpath matches no properties in the graph, the stream will be
	 * empty.
	 *
	 * @param <T>   the expected object type
	 * @param xpath the xpath expression to iterate
	 * @param type  the type of elements in the returned stream
	 * @return a stream of elements matching the specified xpath and of the given
	 *         type
	 * @since 0.5
	 */
	default <T> Stream<T> stream(String xpath, Class<T> type) {
		Iterator<?> iterator = iterate(xpath);
		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, Spliterator.ORDERED), false)
				.filter(e -> TypeUtils.canConvert(e, type)).map(e -> TypeUtils.convert(e, type)).map(type::cast);
	}
}
